import enum

from mrcrowbar import models as mrc


# 00003010: 0025 5043 ffff ffff 00ff ffff ff06 1100  .%PC............
# VFO A: 435.02500, High, Wide, PTT Off, Busy Lock Off
# CTCSS Decode None, CTCSS Encode None, Freq Split Off
# Signaling Code 1, Freq Step 2.5k, Scan Add Greyed Out On, Freq Hopping: Off

# 00003020: 0025 5015 ffff ffff 00ff ffff ff06 1100  .%P.............
# VFO B: 155.02500, High, Wide, PTT Off, Busy Lock Off
# CTCSS Decode None, CTCSS Encode None, Freq Split Off
# Signaling Code 1, Freq Step 2.5k, Scan Add Greyed Out On, Freq Hopping: Off

# 00003030: 5062 2546 5062 2546 00ff ffff ff06 1100  Pb%FPb%F........
# CH1, RX: 462.56250, TX: 462.56250, Rx QD: None, Tx QD: None, Bw: Wide
# Power: High, PTT-ID: Off, Busy Lock: No, Scan Add: Yes
# Signaling Code 1, Freq Hopping: Off

# 000030a0: 5062 7546|5062 7546|00ff ffff ff00 1100  PbuFPbuF........
# CH8, RX: 467.56250, TX: 467.56250, Bw: Narrow, Power: Low
# 46 75 62 50 | 46 75 62 50 | 00 ff ff ff ff | 00 | 11 00
# RX          | TX          | ?? ?? ?? ?? ?? |  BW + Power | ?? ??

# 00003190: 0050 2546|0050 7546|00ff ffff ff06 1100  .P%F.PuF........
# CH23, RX: 462.55000, TX: 467.55000
#               0x25+0x50=0x75
# 46 25 50 00 | 46 75 50 00 | 00 ff ff ff ff | 06 | 11 00


class Frequency(mrc.Block):
    # TODO: this is some weird BCD like encoding where
    value = mrc.Bytes(offset=0x00, length=4)


class Bandwidth(int, enum.Enum):
    WIDE = 0
    NARROW = 1


class Power(int, enum.Enum):
    HIGH = 0
    LOW = 1


class FrequencyEntry(mrc.Block):
    transmit = mrc.BlockField(Frequency, offset=0x00, fill=bytes([0xFF]))
    receive = mrc.BlockField(Frequency, offset=0x04, fill=bytes([0xFF]))

    unknown_entry_flags2 = mrc.Bytes(
        offset=0x08,
        length=5)

    # XXX: confirm this
    bandwidth = mrc.Bits(
        offset=0x0C,
        bits=0b00000100,
        enum=Bandwidth,
        default=Bandwidth.WIDE)

    # XXX: confirm this
    power = mrc.Bits(
        offset=0x0C,
        bits=0x00000010,
        enum=Power,
        default=Power.HIGH)

    # TODO: 0x11 0x00 common for built-in channels
    unknown_entry_flags2 = mrc.Bytes(
        offset=0x0D,
        length=2)


class FrequencyMemory(mrc.Block):
    # TODO: reverse engineer these flags
    unknown_frequency_flags = mrc.Bytes(
        offset=0x00,
        length=0xF,
        default=bytes([0xFA] + ([0x00] * 15)))

    vfo_a = mrc.BlockField(FrequencyEntry, offset=0x10, fill=bytes([0x00]))
    vfo_b = mrc.BlockField(FrequencyEntry, offset=0x20, fill=bytes([0x00]))
    # frequency_entries = mrc.BlockField(
    #     FrequencyEntry,
    #     offset=0x30,
    #     count=250,
    #     # XXX: we really should only consider the frequency value bytes here
    #     fill=bytes([
    #         0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    #         0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0x06, 0x11, 0x00]))

    # XXX: 0xFD0 - 0xFFE are all 0x00
